//
// JigsawServletInputStream.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package w3c.jigsaw.slklet;

import java.io.*;
import java.servlet.*;

class JigsawServletInputStream extends ServletInputStream {
    InputStream in = null;

    public int read()
	throws IOException
    {
	    return in.read();
    }

    public int read(byte b[])
	throws IOException
    {
	    return in.read(b, 0, b.length);
    }

    public int read(byte b[], int off, int len)
	throws IOException
    {
	    return in.read(b, off, len);
    }

    public long skip(long n)
	throws IOException
    {
	    return in.skip(n);
    }

    public int available()
	throws IOException
    {
    	return in.available();
    }

    public void close()
	throws IOException
    {
    	in.close();
    }

    public synchronized void mark(int readlimit) {
    	in.mark(readlimit);
    }

    public synchronized void reset()
	throws IOException
    {
    	in.reset();
    }

    public boolean markSupported() {
	return in.markSupported();
    }

    public int readLine(byte b[], int off, int len)
	throws IOException
    {
    	int got = 0;
    	while (got < len) {
    	    int ch = in.read();
    	    switch(ch) {
    	      case -1:
    		  return -1;
    	      case '\r':
    	      case '\n':
    		  b[off+got] = (byte) (ch&0xff);
    		  got++;
    		  in.mark(1);
    		  if ((ch = in.read()) != '\n' )
    		      in.reset();
    		  return got;
    	      default:
    		  b[off+got] = (byte) (ch&0xff);
    		  got++;
    	    }
    	}
    	return got;
    }

    JigsawServletInputStream(InputStream in) {
    	this.in = in;
    }
}
