//
// JigsawServletRequest.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package w3c.jigsaw.slklet;

import java.io.*;
import java.util.*;

import java.servlet.*;
import java.servlet.http.*;

import w3c.jigsaw.http.*;
import w3c.jigsaw.forms.*;
import w3c.jigsaw.auth.AuthFilter;
import w3c.www.http.*;

// ServletRequest implements Java HttpServletRequest interface

public class JigsawServletRequest implements HttpServletRequest
{
    private Request request;

    JigsawServletRequest(Request request)
    {
        this.request = request;
    }

    // Implementation of ServletRequest Interface

    protected ServletInputStream is = null;
    public ServletInputStream getInputStream()
    {
	    // If alaredy computed return:
	    if ( is != null )
	        return is;
	    // Built it:
	    InputStream stream = null;
        try {
	        if ((stream = request.getInputStream()) != null)
	            return is = new JigsawServletInputStream(stream);
        }
        catch (IOException ex) {
        }
	    return null;
    }

    private Hashtable queryParameters = null;
    private void prepareQueryParameters()
    {
    	if (queryParameters != null) return;
    	String query = request.getQueryString();
    	if (query == null) return ;
    	InputStream qis = new StringBufferInputStream(query);
    	try {
    	    queryParameters = new URLDecoder(qis).newParse();
    	}
       	catch (Exception ex) {
    	    throw new RuntimeException("Java implementation bug.");
    	}
    }

    public int getContentLength()
    {
	    return request.getContentLength();
    }

    public String getContentType()
    {
	    return request.getContentType().toString();
    }

    public String getParameter(String name)
    {
	    prepareQueryParameters();
	    if (queryParameters != null)
            return (String) queryParameters.get(name);
	    else
            return null;
    }

    public Hashtable getParameters()
    {
	    prepareQueryParameters();
	    return queryParameters;
    }

    public String getProtocol()
    {
	    return request.getVersion();
    }

    public String getRemoteAddr()
    {
	    return request.getClient().getInetAddress().toString();
    }

    public String getRemoteHost()
    {
	    return request.getClient().getInetAddress().getHostName();
    }

    public String getServerName()
    {
	    return request.getClient().getServer().getHost();
    }

    public int getServerPort()
    {
	    return request.getClient().getServer().getLocalPort();
    }

    //
    // Implementation of HttpServletRequest Interface
    //

    /**
     * Returns the authentication scheme of the request, or null if
     * none.
     */
    public String getAuthType()
    {
        return (String) request.getState(AuthFilter.STATE_AUTHTYPE);
    }

    /**
     * Returns the value of a date header field
     */
    public long getDateHeader(String name, long def)
    {
        String s = getHeader(name);
        long i = def;

        if (s != null) {
            try {
                return i = Date.parse(s);
            }
            catch (Exception e) {
            }
        }
        return i;

    }

    /**
     * Returns the value of the nth header field, or null if there
     * are fewer than n fields.
     */
    public String getHeader(int n)
    {
        return null;
    }

    /**
     * Returns the value of a header field, or null if not known.
     */
    public String getHeader(String name)
    {
        return request.getValue(name);
    }

    /**
     * Returns the name of the nth header field, or null if there are
     * fewer than n fields.  This can be used to iterate through all
     * the headers in the message.
     */
    public String getHeaderName(int n)
    {
        return null;
    }


    // Returns the value of an integer header field
    public int getIntHeader(String name, int def)
    {
        String s = getHeader(name);
        int i = def;
        if (s != null) {
            try {
                i = Integer.parseInt(s);
            }
            catch (NumberFormatException e) {
            }
        }

        return i;
    }


    // Returns the method with which the request was made, e.g. "GET",
    // "HEAD", "POST", or an extension method.
    public String getMethod()
    {
        return request.getMethod();
    }

    // Returns the optional extra path information following the
    // servlet path but immediately preceding the query string.
    public String getPathInfo()
    {
        return null;
    }

    // Returns the extra path information translated to a real path
    public String getPathTranslated()
    {
        return (String) null;
    }


    // Returns the query string part of the servlet URL, or null if
    // none
    public String getQueryString()
    {
        return (String) null;
    }

    // Returns the name of the user making this request, or null if
    // not known.

    public String getRemoteUser()
    {
        return (String) request.getState(AuthFilter.STATE_AUTHUSER);
    }

    // Returns the part of the requested URI that corresponds to the
    // servlet path plus the optional extra path information, if any.
    public String getRequestPath()
    {
        return request.getURLPath();
    }


    // Returns the request URI
    public String getRequestURI()
    {
        return request.getURL().toExternalForm();
    }

    // Returns the part of the request URI that refers to the servlet
    // being invoked
    public String getServletPath()
    {
        return request.getTargetResource().getURLPath();
    }

}
