//
// ServletDirectory.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package w3c.jigsaw.slklet;

import java.util.*;
import java.io.*;
import java.servlet.*;

import w3c.jigsaw.resources.*;


public class ServletDirectory extends DirectoryResource implements ServletContext
{
    /**
     * A useful utility routine that tries to guess the content-type
     * of an object based upon its execution.
     */

    protected static String guessContentTypeFromName(String fname)
    {
        String ext = "";
        int i = fname.lastIndexOf('#');

        if (i != -1)
            fname = fname.substring(0, i - 1);
        i = fname.lastIndexOf('.');
        i = Math.max(i, fname.lastIndexOf('/'));
        i = Math.max(i, fname.lastIndexOf('?'));

        if (i != -1 && fname.charAt(i) == '.') {
            ext = fname.substring(i).toLowerCase();
        }
        return (String) extension_map.get(ext);
    }

    static Hashtable extension_map = new Hashtable();

    static {
        setSuffix("", "content/unknown");
        setSuffix(".uu", "application/octet-stream");
        setSuffix(".saveme", "application/octet-stream");
        setSuffix(".dump", "application/octet-stream");
        setSuffix(".hqx", "application/octet-stream");
        setSuffix(".arc", "application/octet-stream");
        setSuffix(".o", "application/octet-stream");
        setSuffix(".a", "application/octet-stream");
        setSuffix(".bin", "application/octet-stream");
        setSuffix(".exe", "application/octet-stream");
        /* Temporary only. */
        setSuffix(".z", "application/octet-stream");
        setSuffix(".gz", "application/octet-stream");

        setSuffix(".oda", "application/oda");
        setSuffix(".pdf", "application/pdf");
        setSuffix(".eps", "application/postscript");
        setSuffix(".ai", "application/postscript");
        setSuffix(".ps", "application/postscript");
        setSuffix(".rtf", "application/rtf");
        setSuffix(".dvi", "application/x-dvi");
        setSuffix(".hdf", "application/x-hdf");
        setSuffix(".latex", "application/x-latex");
        setSuffix(".cdf", "application/x-netcdf");
        setSuffix(".nc", "application/x-netcdf");
        setSuffix(".tex", "application/x-tex");
        setSuffix(".texinfo", "application/x-texinfo");
        setSuffix(".texi", "application/x-texinfo");
        setSuffix(".t", "application/x-troff");
        setSuffix(".tr", "application/x-troff");
        setSuffix(".roff", "application/x-troff");
        setSuffix(".man", "application/x-troff-man");
        setSuffix(".me", "application/x-troff-me");
        setSuffix(".ms", "application/x-troff-ms");
        setSuffix(".src", "application/x-wais-source");
        setSuffix(".wsrc", "application/x-wais-source");
        setSuffix(".zip", "application/zip");
        setSuffix(".bcpio", "application/x-bcpio");
        setSuffix(".cpio", "application/x-cpio");
        setSuffix(".gtar", "application/x-gtar");
        setSuffix(".shar", "application/x-shar");
        setSuffix(".sh", "application/x-shar");
        setSuffix(".sv4cpio", "application/x-sv4cpio");
        setSuffix(".sv4crc", "application/x-sv4crc");
        setSuffix(".tar", "application/x-tar");
        setSuffix(".ustar", "application/x-ustar");
        setSuffix(".snd", "audio/basic");
        setSuffix(".au", "audio/basic");
        setSuffix(".aifc", "audio/x-aiff");
        setSuffix(".aif", "audio/x-aiff");
        setSuffix(".aiff", "audio/x-aiff");
        setSuffix(".wav", "audio/x-wav");
        setSuffix(".gif", "image/gif");
        setSuffix(".ief", "image/ief");
        setSuffix(".jfif", "image/jpeg");
        setSuffix(".jfif-tbnl", "image/jpeg");
        setSuffix(".jpe", "image/jpeg");
        setSuffix(".jpg", "image/jpeg");
        setSuffix(".jpeg", "image/jpeg");
        setSuffix(".tif", "image/tiff");
        setSuffix(".tiff", "image/tiff");
        setSuffix(".ras", "image/x-cmu-rast");
        setSuffix(".pnm", "image/x-portable-anymap");
        setSuffix(".pbm", "image/x-portable-bitmap");
        setSuffix(".pgm", "image/x-portable-graymap");
        setSuffix(".ppm", "image/x-portable-pixmap");
        setSuffix(".rgb", "image/x-rgb");
        setSuffix(".xbm", "image/x-xbitmap");
        setSuffix(".xpm", "image/x-xpixmap");
        setSuffix(".xwd", "image/x-xwindowdump");
        setSuffix(".htm", "text/html");
        setSuffix(".html", "text/html");
        setSuffix(".text", "text/plain");
        setSuffix(".c", "text/plain");
        setSuffix(".cc", "text/plain");
        setSuffix(".c++", "text/plain");
        setSuffix(".h", "text/plain");
        setSuffix(".pl", "text/plain");
        setSuffix(".txt", "text/plain");
        setSuffix(".java", "text/plain");
        setSuffix(".rtx", "application/rtf");
        setSuffix(".tsv", "text/tab-separated-values");
        setSuffix(".etx", "text/x-setext");
        setSuffix(".mpg", "video/mpeg");
        setSuffix(".mpe", "video/mpeg");
        setSuffix(".mpeg", "video/mpeg");
        setSuffix(".mov", "video/quicktime");
        setSuffix(".qt", "video/quicktime");
        setSuffix(".avi", "application/x-troff-msvideo");
        setSuffix(".movie", "video/x-sgi-movie");
        setSuffix(".mv", "video/x-sgi-movie");
        setSuffix(".mime", "message/rfc822");
    }

    static private void setSuffix(String ext, String ct)
    {
        extension_map.put(ext, ct);
    }



    /**
     * ServletContext implementation - Get the MIME type for given file.
     */
    public String getMimeType(String filename)
    {
        return guessContentTypeFromName(filename);
    }

    /**
     * ServletContext implementation - Lookup a given servlet.
     */
    public Servlet getServlet(String name)
    {
        // Jigsaw alpha5 ContainerResource throws an InvalidResourceException
        // try {

        Resource resource = lookup(name);
        return ((resource instanceof ServletResource)
            ? ((ServletResource) resource).getServletWrapper()
            : null);

        // }
        // catch (InvalidResourceException e) {
        //     return null;
        // }
    }

    /**
     * ServletContext implementation - Enumerate all servlets within context.
     */
    public Enumeration getServlets()
    {
        return new ServletEnumeration(this, enumerateResourceIdentifiers());
    }

    /**
     * ServletContext implementation - Log a message.
     */
    public void log(Servlet servlet, String msg)
    {
        System.out.println(msg);
    }

    /**
     * ServletContext implementation - Translate a piece of path.
     */
    public String getRealPath(String path)
    {
        return path;
    }

    /**
     * ServletContext Implementation - Returns the name and version of the
     * Web server under which the server is running.
     */

    public String getServerInfo()
    {
        return getServer().getSoftware();
    }
}
