//
// ServletEnumeration.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package w3c.jigsaw.slklet;

import java.util.*;
import java.servlet.*;

/**
 *  @author Alexandre Rafalovitch <alex@access.com.au>
 *  @author Anselm Baird-Smith <abaird@w3.org>
 */

public class ServletEnumeration implements Enumeration
{
    Enumeration      children = null;
    Servlet          next     = null;
    ServletDirectory dir      = null;

    ServletEnumeration(ServletDirectory dir, Enumeration children)
    {
	    this.dir      = dir;
    	this.children = null;
    }

    private final synchronized Servlet computeNext()
    {
    	if (next != null) return next;
    	while (children.hasMoreElements()) {
    	    next = dir.getServlet((String) children.nextElement());
    	    if (next != null)	return next;
    	}
    	return null;
    }

    public synchronized boolean hasMoreElements()
    {
    	return (next != null) || ((next = computeNext()) != null);
    }

    public synchronized Object nextElement()
    {
    	if (next == null) next = computeNext();
    	if ( next != null ) {
    	    Object ret = next;
    	    next = null;
    	    return ret;
    	}
    	else {
    	    throw new NoSuchElementException("NextElement");
    	}
    }
}
