//
// ServletResource.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package w3c.jigsaw.slklet;

import java.lang.*;
import java.io.*;
import java.net.*;
import java.servlet.*;


public class ServletLoader extends ClassLoader
{
    private static final boolean debug = false;
    ServletStub servletStub = null;
    ServletResource servletRes = null;

    ServletLoader(ServletResource resource)
    {
        super ();
        servletRes = resource;
    }

    protected boolean servletIsLocal()
    {
        String base = servletRes.getServletBase();
        return (base == null) ? true : false;
    }

    protected URL locateServlet(String name)
    {
        String base = servletRes.getServletBase();
        try {
            String cname = name.replace('.','/') + ".class";
            return new URL(new URL(base), cname);
        }
        catch (Exception e) {
        }
        return null;
    }

    public Class loadClass(String name, boolean resolve)
    throws ClassNotFoundException
    {

        Class c = null;
        byte[] buf = null;


        // First check if the system class loader already has the class
        // loaded.  If so, ask it to give you a reference to it.

        try {
            if ((c = findSystemClass(name)) != null) {
                if (debug) {
                    System.out.println(name + " loaded from system class loader");
                }
                return c;
            }
        }
        catch (Exception e) {
        }

        // FIXME:
        // If not, check if servlet is local.  A local servlet has a null
        // servlet base attribute (default value).  If the base attribute
        // is not null, then the servlet is treated as a remote one.

        if (servletIsLocal()) {

            // Load the class file from disk.

            String cname = name.replace('.', '/') + ".class";
            buf = readClassFromFile(cname);
            if (buf == null) {
                throw new ClassNotFoundException("Local servlet not found");
            }
            if (debug) {
                System.out.println(name + " loaded from " + cname);
            }
        }
        else {
            buf = readClassFromNet(name);
        }

        c = defineClass(buf, 0, buf.length);
        if (resolve) {
            resolveClass(c);
        }
        return c;
    }


    protected byte[] readClassFromFile(String path)
    {
        byte[] b = null;
        int available;

        try {
            InputStream is = new FileInputStream(path);
            available = is.available();
            b = new byte[available];
            is.read(b);
        }
        finally {
            return b;
        }
    }

    protected byte[] readClassFromNet(String name)
    throws ClassNotFoundException
    {
        byte[] data = null;
        URL url = locateServlet(name);
        if (url == null) {
            throw new ClassNotFoundException("invalid servlet base");
        }

        if (debug) {
            System.out.println("Loading " + name + " from " + url);
        }
        try {
            URLConnection conn = url.openConnection();
            InputStream in = conn.getInputStream();
            ByteArrayOutputStream out = new ByteArrayOutputStream(512);
            byte buf[] = new byte[512];
            for (int got = 0; (got = in.read(buf)) > 0; )
            out.write(buf, 0, got);
            data = out.toByteArray();
            in.close();
        }
        catch (Exception e) {
            e.printStackTrace();
            throw new ClassNotFoundException(url.toExternalForm());
        }
        if (debug) {
            System.out.println("Loaded");
        }

        return data;
    }

}
