//
// ServletWrapperCCP.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package w3c.jigsaw.slklet;

import SLK.*;
import java.io.*;
import java.servlet.*;

/**
 * ServletWrapperCCP implements a SLK wrapper with caller/callee thread
 * protection. This serves as a sample output CallerCalleeProtected
 * wrapper output for an automatic generator.
 */

public class ServletWrapperCCP extends GenericServletWrapper
{
    private final static boolean debug = false;

    public ServletWrapperCCP(Servlet servlet)
    {
        super(servlet);
    }

    public final void destroy()
    {
        SlkThread currentThread = (SlkThread) Thread.currentThread();
        ThreadRights prevRights = currentThread.currentRights;

        try {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = null;
            }

            servlet.destroy();
        }
        finally {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = prevRights;
            }
        }
    }

    public final String getServletInfo()
    {
        SlkThread currentThread = (SlkThread) Thread.currentThread();
        ThreadRights prevRights = currentThread.currentRights;

        try {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = null;
            }
            return servlet.getServletInfo();
        }
        finally {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = prevRights;
            }
        }
    }

    public final void init(ServletStub stub)
    throws ServletException
    {
        SlkThread currentThread = (SlkThread) Thread.currentThread();
        ThreadRights prevRights = currentThread.currentRights;

        try {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = null;
            }
            servlet.init(stub);
        }
        finally {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = prevRights;
            }
        }
    }

    public final void service(ServletRequest request, ServletResponse response)
    throws ServletException, IOException
    {
        SlkThread currentThread = (SlkThread) Thread.currentThread();
        ThreadRights prevRights = currentThread.currentRights;

        try {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = null;
            }
            servlet.service(request, response);
        }
        finally {
            synchronized (currentThread.switchLock) {
                currentThread.currentRights = prevRights;
            }
        }

    }
}