//
// ServletWrapperTS.java
//
// Copyright (c) 1997 by Cornell University
// All rights reserved.
//
// Permission is hereby granted, without written agreement and without
// license or royalty fee, to use, copy, modify, and distribute this
// software and its documentation for any purpose, provided that the
// above copyright notice and the following two paragraphs appear in
// all copies of this software
//
// IN NO EVENT SHALL CORNELL UNIVERSITY BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
// IF CORNELL UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGE.
//
// CORNELL UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
// BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
// ON AN "AS IS" BASIS, AND CORNELL UNIVERSITY HAS NO OBLIGATION TO
// PROVIDE MAINTAINANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//

package w3c.jigsaw.slklet;

import java.io.*;
import java.servlet.*;

/**
 * ServletWrapperTS implements a SLK wrapper where the caller and callee
 * have separate threads. This serves as a sample ThreadSwitching
 * wrapper output for an automatic generator.
 */

public class ServletWrapperTS extends GenericServletWrapper implements Runnable
{
    private final static boolean debug = false;

    public ServletWrapperTS(Servlet servlet)
    {
        super(servlet);

        //
        // Put the thread group stuff here.
        //

        if (workerThread == false) {
            Thread t = new Thread(this);
            t.start();
            Thread.currentThread().yield();
            workerThread = true;
        }
    }

    private static boolean workerThread = false;


    private boolean done = false;
    private int whichCall = -1;
    private final Object sa = new Object();
    private final Object sb = new Object();

    //
    // DESTROY
    //

    private final int DESTROY = 0;
    public final void destroy()
    {
        try {
            synchronized (sa) {
                done = false;
                whichCall = DESTROY;
                sa.notify();
            }
            synchronized (sb) { while (!done) sb.wait(); }
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }
    }

    //
    // INFO
    //
    private final int INFO = 1;
    private static String infoRet = null;

    public final String getServletInfo()
    {
        try {
            synchronized (sa) {
                done = false;
                whichCall = INFO;
                sa.notify();
            }
            synchronized (sb) { while (!done) sb.wait(); }
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }
        return infoRet;
    }

    //
    // INIT
    //
    private final int INIT = 2;
    private static ServletStub initArg1 = null;
    private static ServletException initServletException = null;

    public final void init(ServletStub stub)
    throws ServletException
    {
        try {
            synchronized (sa) {
                done = false;
                initArg1 = stub;
                initServletException = null;
                whichCall = INIT;
                sa.notify();
            }
            synchronized (sb) { while (!done) sb.wait(); }
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }
        if (initServletException != null) throw initServletException;
    }

    //
    // SERVICE
    //
    private final int SERVICE = 3;
    private static ServletRequest serviceArg1 = null;
    private static ServletResponse serviceArg2 = null;
    private static ServletException serviceServletException = null;
    private static IOException serviceIOException = null;
    public final void service(ServletRequest request, ServletResponse response)
    throws ServletException
    {
        try {
            synchronized (sa) {
                done = false;
                serviceArg1 = request;
                serviceArg2 = response;
                whichCall = SERVICE;
                sa.notify();
            }
            synchronized (sb) { while (!done) sb.wait(); }
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }

        if (serviceServletException != null) throw serviceServletException;

    }

    //
    // RUN
    //

    public void run()
    {
        try {
            synchronized (sa) {
                while (true) {
                    sa.wait();
                    switch(whichCall) {
                        case DESTROY:
                            servlet.destroy();
                            break;

                        case INFO:
                            infoRet = servlet.getServletInfo();
                            break;

                        case INIT:
                            try {
                                servlet.init(initArg1);
                            }
                            catch (ServletException e) {
                                initServletException = e;
                            }
                            break;

                        case SERVICE:
                            try {
                                servlet.service(serviceArg1, serviceArg2);
                            }
                            catch (ServletException e) {
                                serviceServletException = e;
                            }
                            catch (IOException ioe) {
                                serviceIOException = ioe;
                            }
                            break;

                        default:
                            System.out.println("Unknown call.");

                    }
                    done = true;
                    synchronized (sb) { sb.notify(); }

                }
            }
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }
    }
}